#include <Arduino.h>

#include "SPI.h"
#include "TFT_eSPI.h"
#include "touchScreenPoint.h"
#include "../lvgl/src/lvgl.h"

#define HOR_RES 320
#define VER_RES 240

TFT_eSPI tft = TFT_eSPI();

TouchScrPoint oldPoint;

static lv_disp_buf_t disp_buf; //LVGL stuff;
static lv_color_t buf[LV_HOR_RES_MAX * 10];
lv_disp_drv_t disp_drv;
lv_indev_drv_t indev_drv;

lv_obj_t * btn1;
lv_obj_t * btn2;
lv_obj_t *screenMain;
lv_obj_t *label;

TouchScrPoint tftTouchGetPoint(){
  uint16_t x, y;
  TouchScrPoint tsp(-1,-1);

  if (tft.getTouch(&x, &y)) {
    tsp = TouchScrPoint(x,y);
  }

  return tsp;
}

static void event_handler_btn(lv_obj_t * obj, lv_event_t event){
  if(event == LV_EVENT_CLICKED) {
    if (obj == btn1)
      lv_label_set_text(label, "Hola");
    else 
      if (obj == btn2){
        lv_label_set_text(label, "A10");
      }
  }
}

void my_disp_flush(lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p){
  uint32_t w = (area->x2 - area->x1 + 1);
  uint32_t h = (area->y2 - area->y1 + 1);

  tft.startWrite();
  tft.setAddrWindow(area->x1, area->y1, w, h);
  tft.pushColors(&color_p->full, w * h, true);
  tft.endWrite();

  lv_disp_flush_ready(disp);
}

bool my_input_read(lv_indev_drv_t * drv, lv_indev_data_t*data){
  uint16_t x, y;

  if (tft.getTouch(&x, &y)){
    data->state = LV_INDEV_STATE_PR;
    data->point.x = oldPoint.x = x;
    data->point.y = oldPoint.y = y;
    String sz = "x: " + String(x,DEC) + " , y: " + String(y,DEC);
    Serial.println(sz);   
  }else{
    data->state = LV_INDEV_STATE_REL;
    data->point.x = oldPoint.x;
    data->point.y = oldPoint.y;    
  }

  return false; 
}

void setup() {
  Serial.begin(115200);

  tft.begin();
  tft.setRotation(1);
  oldPoint = tftTouchGetPoint();

  lv_init();
  lv_disp_buf_init(&disp_buf, buf, NULL, LV_HOR_RES_MAX * 10);
  /*Initialize the display*/
  lv_disp_drv_init(&disp_drv);
  disp_drv.hor_res = HOR_RES;
  disp_drv.ver_res = VER_RES;
  disp_drv.flush_cb = my_disp_flush;
  disp_drv.buffer = &disp_buf;
  lv_disp_drv_register(&disp_drv);

  /*Initialize the input device driver*/
  lv_indev_drv_init(&indev_drv);             /*Descriptor of a input device driver*/
  indev_drv.type = LV_INDEV_TYPE_POINTER;    /*Touch pad is a pointer-like device*/
  indev_drv.read_cb = my_input_read;         /*Set your driver function*/
  lv_indev_drv_register(&indev_drv);         /*Finally register the driver*/

  screenMain = lv_obj_create(NULL, NULL);

  label = lv_label_create(screenMain, NULL);
  lv_label_set_long_mode(label, LV_LABEL_LONG_BREAK);
  lv_label_set_text(label, "Premeu-ne un");
  lv_label_set_align(label, LV_LABEL_ALIGN_CENTER);
  lv_obj_set_size(label, HOR_RES, 40);
  lv_obj_set_pos(label, 0, 15);

  btn1 = lv_btn_create(screenMain, NULL);
  lv_obj_set_event_cb(btn1, event_handler_btn);
  lv_obj_set_width(btn1, 70);
  lv_obj_set_height(btn1, 32);
  lv_obj_set_pos(btn1, 52, 130);

  lv_obj_t * label1 = lv_label_create(btn1, NULL);
  lv_label_set_text(label1, "Hola");

  btn2 = lv_btn_create(screenMain, NULL);
  lv_obj_set_event_cb(btn2, event_handler_btn);
  lv_obj_set_width(btn2, 70);
  lv_obj_set_height(btn2, 32);
  lv_obj_set_pos(btn2, 162, 130);

  
  lv_obj_t * label2 = lv_label_create(btn2, NULL);
  lv_label_set_text(label2, "A10");

  static lv_style_t style1;
  lv_style_init(&style1);
  lv_style_set_text_color(&style1, LV_STATE_DEFAULT, LV_COLOR_GREEN);
  lv_style_set_border_color(&style1, LV_STATE_DEFAULT, LV_COLOR_RED);
  lv_style_set_border_width(&style1, LV_STATE_DEFAULT, 2);
  lv_style_set_bg_color(&style1, LV_STATE_DEFAULT, LV_COLOR_YELLOW);
  lv_style_set_bg_opa(&style1, LV_STATE_DEFAULT, 255);
  lv_style_set_radius(&style1, LV_STATE_DEFAULT, 1);
  
  static lv_style_t style2;
  lv_style_init(&style2);
  lv_style_set_text_color(&style2, LV_STATE_DEFAULT, LV_COLOR_GRAY);
  lv_style_set_border_color(&style2, LV_STATE_DEFAULT, lv_color_hex(0x222222));
  lv_style_set_border_width(&style2, LV_STATE_DEFAULT, 1);
  lv_style_set_bg_color(&style2, LV_STATE_DEFAULT, lv_color_hex(0x222222));
  lv_style_set_bg_opa(&style2, LV_STATE_DEFAULT, 255);
  lv_style_set_radius(&style2, LV_STATE_DEFAULT, 4);
  lv_style_set_shadow_spread(&style2, LV_STATE_DEFAULT, 1);
  lv_style_set_shadow_color(&style2, LV_STATE_DEFAULT, LV_COLOR_GRAY);
  lv_style_set_shadow_opa(&style2, LV_STATE_DEFAULT, 255);
  lv_style_set_shadow_width(&style2, LV_STATE_DEFAULT, 1);

  static lv_style_t bigStyle;
  lv_style_init(&bigStyle);
  lv_style_set_text_font(&bigStyle ,LV_STATE_DEFAULT, &lv_font_montserrat_36);

  lv_obj_add_style(label, LV_LABEL_PART_MAIN, &bigStyle);

  //Uncomment to chabge buutton 1 styling:
  // lv_obj_add_style(btn1, LV_LABEL_PART_MAIN, &style1);

  //Uncomment to chabge buutton 2 styling:
  //lv_obj_add_style(btn2, LV_LABEL_PART_MAIN, &style2);
  
  lv_scr_load(screenMain);
}

void loop() {
  lv_task_handler();

  delay(1);
}